

# AutoForm Component

the auto form component is part of the auto-views component library, it renders a form from a schema.

![image](https://user-images.githubusercontent.com/2289769/27994053-88ffa818-64be-11e7-8301-b9387e721eab.png)


* [Properties](#properties)
* [style](#style)
* [handling input](#Handling-Input)


## Properties

| Name | Type | Default | Required | Description |
| -- | -- | -- | -- | -- |
| schemaId | string | undefined | No | the id of the schema which will define the view |
| schemaRepo| SchemaRepo | empty SchemaRepo | No | a repository of the schemas to be used in constructing the view |
| view | string | "edit" | No | the name of the view to request from the repo |
| value | object| null | No | data for the form, needs to comply with schema |  expanded/collapsed directly (search can do it) or traversed using keyboard, or selected. |
| onChange | (ev:[AutoViewEvent](./auto-view-event.md))=>void | none | No| callback method for data change |


## Style

Supports the following style states for the form root and all internal parts:
* hover
* valid
* invalid
* empty
* focused

### internal parts
* field
* label
* form-group - input and label container
* ui-group - fields grouped together by schema
* sub-form

each of the internal parts also gets a cssState with its respective field/group name

```jsx
<input cssStates={{name:fieldName}} className="field"/>

```

example styling


```css
.myForm{
    background:blue;
}
//applies to the root of the form

.myForm:invalid{
    background:red;
}
//applies to the root of the form when its invalid

.myForm::field{
    background:yellow;
}
// applies to all fields

.myForm::field:name(title):empty{
    background:blue;
}

// applies to the title field if its empty

```


## Handling-Input

### Keyboard Navigation

* Home -> focuses the first field in the AutoForm
* End -> focuses the last field in the AutoForm
* Enter -> focuses the next field if available
* Tab -> focuses the next field if available

## Stages of development

* Render a form with one field
* Render a form with complex field ( recursive creation  )
* Support onChange with full data snapshot
* Support onChange with JSON Patch

