# Generating a complex view
In this example we will demonstrate how schames can be used to genereate complex interfaces, using hints such as `uiGroup` and `icon`.

## Schema
``` javascript
const userSchema = {
  type: 'object',
  title: 'user',
  properties: {
    fname: {
      title: 'First name',
      type: 'string',
      uiGroup: 'general'
    },
    lname: {
      title: 'Last name',
      type: 'string',
      uiGroup: 'general'
    },
    email: {
      title: 'Email',
      type: 'email',
      uiGroup: 'actions'
    },
    avatar: {
      title: 'Avatar',
      $ref: 'http://example.com/MpImage',
      description: 'Wix media platform image',
      uiGroup: 'avatar',
      views: {
        view: {
          controller: 'MpImageViewer',
          props: {
            cropMode: 'Center'
          }
        }
      }
    },
    description: {
      title: 'Description',
      type: 'string',
      uiGroup: 'general'
    },
    following: {
      title: 'Following',
      type: 'array',
      items: {
        $ref: 'http://example.com/user'
      },
      uiGroup: 'links',
      size: 's'
    },
    followers: {
      title: 'Followers',
      type: 'array',
      items: {
        $ref: 'http://example.com/user'
      },
      uiGroup: 'links',
      size: 's'
    },
    dateJoined: {
      type: date,
      title: 'Date joined   '
    },
    location: {
      title: 'Location',
      type: 'string',
      uiGroup: 'links',
      icon: 'location/map-pin'
    },
    website: {
      title: 'Website',
      type: url,
      uiGroup: 'links',
      icon: 'globe'
    },
    gallery: {
      title: 'Posts',
      type: 'array',
      uiGroup: 'content',
      items: {
        $ref: 'http://example.com/MpImage'
      }
    }
  }
}

```

As you can see, the schema represents a single user. We could render this data a few ways.

### Read Only
![image](https://user-images.githubusercontent.com/2289769/28119114-5ab7af5e-671c-11e7-8bd4-d5ac3355cc5c.png)

### DOM
![image](https://user-images.githubusercontent.com/2289769/28117423-09013230-6716-11e7-9055-9f8f833f89a4.png)


## Lists
Besides showing or editing a single `User`, we could use different generated views to handle arrays of the same data

### List Editor
![image](https://user-images.githubusercontent.com/2289769/28119144-7331a09e-671c-11e7-928e-9720e09d4c5b.png)

### Table View
![image](https://user-images.githubusercontent.com/2289769/28119621-2d5c1016-671e-11e7-8837-445d8ef5a587.png)
