"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var schema_repo_1 = require("./schema-repo");
var form_entry_1 = require("./form-entry");
var json_patch_builder_1 = require("./json-patch-builder");
function newSchemaRepository() {
    return new schema_repo_1.SchemaRepository();
}
exports.newSchemaRepository = newSchemaRepository;
function fieldValueSelector(key) {
    return "form_value_" + key;
}
exports.fieldValueSelector = fieldValueSelector;
function fieldLabelSelector(key) {
    return "form_label_" + key;
}
exports.fieldLabelSelector = fieldLabelSelector;
function autoFormSelector(key) {
    return "auto_form_" + key;
}
exports.autoFormSelector = autoFormSelector;
var AutoForm = (function (_super) {
    __extends(AutoForm, _super);
    function AutoForm(props) {
        var _this = _super.call(this, props) || this;
        _this.onAutoFormChange = function (e) {
            if (_this.props.onAutoViewChange) {
                var patch = e.jsonPatch;
                if (patch) {
                    var propertyName = e.key;
                    patch.prepandToPath(propertyName, _this.props.formData[propertyName]);
                    _this.props.onAutoViewChange({
                        jsonPatch: patch,
                        key: _this.props.entryKey,
                    });
                }
                else {
                    var patch_1 = new json_patch_builder_1.JsonPatchBuilder();
                    patch_1.initPath(e.key, e.value, _this.props.formData);
                    _this.props.onAutoViewChange({
                        jsonPatch: patch_1,
                        key: _this.props.entryKey,
                    });
                }
            }
        };
        return _this;
    }
    AutoForm.prototype.getChildContext = function () {
        return { isInternal: true };
    };
    AutoForm.prototype.renderChildren = function () {
        if (!this.props.repo) {
            return [AutoForm.errorLabel()];
        }
        else {
            var validationErrors = this.props.repo.validate(this.props.formData, this.props.schemaId).errors;
            var erroneousProps = validationErrors.map(function (err) { return err.schema; });
            return this.props.repo.mapOverProperties(this.props.schemaId, makeFormEntry(this.props.formData, this.props.displayMode, this.props.repo, erroneousProps, this.onAutoFormChange));
        }
    };
    AutoForm.errorLabel = function () {
        return React.createElement("label", { "data-automation-id": "errMessage", key: "errMessage" }, "can't render form - missing properties");
    };
    AutoForm.prototype.render = function () {
        var _a = this.props, repo = _a.repo, schemaId = _a.schemaId, displayMode = _a.displayMode, formData = _a.formData, entryKey = _a.entryKey, onAutoViewChange = _a.onAutoViewChange, rest = __rest(_a, ["repo", "schemaId", "displayMode", "formData", "entryKey", "onAutoViewChange"]); //remove props that don't need to pass to div
        var children = this.renderChildren();
        if (!this.context.isInternal) {
            return React.createElement("form", __assign({}, rest), children);
        }
        else {
            return React.createElement("fieldset", __assign({}, rest), children);
        }
    };
    AutoForm.defaultProps = {
        entryKey: "ROOT",
    };
    AutoForm.contextTypes = {
        isInternal: function () { return null; },
    };
    AutoForm.childContextTypes = {
        isInternal: function () { return null; },
    };
    return AutoForm;
}(React.Component));
exports.AutoForm = AutoForm;
function makeFormEntry(formData, displayMode, repo, erroneousProps, onAutoFormChange) {
    if (erroneousProps === void 0) { erroneousProps = []; }
    return function (property, key) {
        var isValid = (erroneousProps.find(function (prop) {
            return prop == property;
        })) == undefined;
        var schemaId = "";
        if (property.type == 'object') {
            schemaId = property['$ref'];
        }
        if (property.type == 'array' && property.items && property.items.type == 'object') {
            schemaId = property.items['$ref'];
        }
        return React.createElement(form_entry_1.Entry, { repo: repo, schemaId: schemaId, "data-automation-id": form_entry_1.entrySelector(key), key: key, entryKey: key, title: property.title || "", displayMode: displayMode, type: property.type, value: formData[key], onChange: onAutoFormChange, valid: isValid });
    };
}
exports.makeFormEntry = makeFormEntry;
//# sourceMappingURL=auto-form.js.map