"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var jsonschema_1 = require("jsonschema");
var SchemaRepository = (function () {
    function SchemaRepository() {
        this.schemas = {};
        this.controllers = {};
        this.validator = new jsonschema_1.Validator();
    }
    SchemaRepository.prototype.addSchema = function (schema) {
        this.schemas[schema.id] = schema;
        this.validator.addSchema(schema);
    };
    SchemaRepository.prototype.addSchemas = function (schemas) {
        var _this = this;
        schemas.forEach(function (schema) { return _this.addSchema(schema); });
    };
    SchemaRepository.prototype.getSchema = function (id) {
        return this.schemas[id];
    };
    SchemaRepository.prototype.clear = function () {
        this.schemas = {};
        this.validator = new jsonschema_1.Validator();
    };
    SchemaRepository.prototype.mapOverProperties = function (id, propertyVisitor) {
        var schema = this.getSchema(id);
        var schemaProperties = schema ? schema.properties : {};
        return Object.keys(schemaProperties).map(function (property) { return propertyVisitor(schemaProperties[property], property); });
    };
    SchemaRepository.prototype.registerController = function (controllerID, controller) {
        this.controllers[controllerID] = controller;
    };
    SchemaRepository.prototype.getController = function (controllerID) {
        return this.controllers[controllerID];
    };
    SchemaRepository.prototype.validate = function (formData, schemaId) {
        return this.validator.validate(formData, this.getSchema(schemaId));
    };
    return SchemaRepository;
}());
exports.SchemaRepository = SchemaRepository;
//# sourceMappingURL=schema-repo.js.map