import * as React from 'react';
import {fieldLabelSelector, fieldValueSelector, AutoFormOnChangeType} from "./auto-form";
import {EditSkin, ViewSkin} from "./skin";
import {Type} from "./schema-property";
import {SchemaRepository} from "./schema-repo";
import {getNonLocalChildProps} from "./component-renderer";
import {JsonPatchBuilder} from "./json-patch-builder";

export enum DisplayMode {
    View="View",Edit="Edit"
}

export interface AutoFormEvent{
    jsonPatch: JsonPatchBuilder,
    value?: any,
    key:string,
}

function isFormEvent(e: AutoViewEvent):e is React.FormEvent<HTMLInputElement> {
    if ((e as React.FormEvent<HTMLInputElement>).currentTarget){
        return true;
    }
    return false;
}

export interface EntryFormEvent<T> extends React.FormEvent<T> {
    value: any,
    key:string,
}

export type AutoViewEvent = React.FormEvent<HTMLInputElement> | AutoFormEvent;

export function entrySelector(key: string):string {
    return `entry_value_${key}`;
}

export interface EntryProps {
    repo:SchemaRepository,
    schemaId:string,
    entryKey:string;
    title:string;
    displayMode:DisplayMode;
    type:Type;
    value?:any;
    onChange?:AutoFormOnChangeType;
    valid: boolean;
}

export class Entry extends React.Component<EntryProps, {}> {
    public static defaultProps = {
        displayMode: DisplayMode.View,
    };
    private renderByDisplayMode: any;

    constructor(props:EntryProps){
        super(props);
        this.renderByDisplayMode = {
            [DisplayMode.Edit] : new EditSkin(),
            [DisplayMode.View] : new ViewSkin()
        };
    }

    private onChange = (e: AutoViewEvent):void=>{
        if (this.props.onChange){
            if (isFormEvent(e)){
                this.props.onChange({
                    value: e.currentTarget.value,
                    key:this.props.entryKey,
                } as any);  //TODO its not really good
            }else{
                this.props.onChange(e);
            }
        }
    };

    private renderInternalComponent():JSX.Element {
        let childProps = Object.assign({},
            {...this.props},
            {['data-automation-id']: fieldValueSelector(this.props.entryKey)},
            (this.props.valid) ? {} : {style: {borderColor: 'red'}},
            (this.props.onChange) ? {onChange:this.onChange}:{});


        let skin: any = this.renderByDisplayMode[this.props.displayMode];
        return skin[this.props.type](childProps);
    }

    render() {
        const rest = getNonLocalChildProps(this.props);
        let comp = this.renderInternalComponent();
        return <div {...rest}>
            <label data-automation-id={fieldLabelSelector(this.props.entryKey)}>{this.props.title}</label>
            {comp}
        </div>
    }
}

