import * as React from 'react';
import {Validator, ValidatorResult} from 'jsonschema';
import {SchemaProperty} from "./schema-property";

export interface AutoFormSchema {
    id:string;
    properties:{[key:string]:SchemaProperty};
}

export class SchemaRepository {
    schemas:{[key:string]:AutoFormSchema} = {};
    controllers: {[key:string]:React.ComponentType} = {};
    validator:Validator = new Validator();

    addSchema(schema:AutoFormSchema):void {
        this.schemas[schema.id] = schema;
        this.validator.addSchema(schema);
    }

    addSchemas(schemas:AutoFormSchema[]):void {
        schemas.forEach((schema)=>this.addSchema(schema));
    }

    getSchema(id:string):AutoFormSchema {
        return this.schemas[id];
    }

    clear() {
        this.schemas = {};
        this.validator = new Validator();
    }

    mapOverProperties<T>(id: string, propertyVisitor: (property: SchemaProperty, key:string) => T):Array<T> {
        let schema = this.getSchema(id);
        const schemaProperties = schema ? schema.properties : {};
        return Object.keys(schemaProperties).map((property)=> propertyVisitor(schemaProperties[property],property));
    }

    registerController(controllerID: string, controller: React.ComponentType):void {
        this.controllers[controllerID] = controller;
    }

    getController(controllerID: string): React.ComponentType | null {
        return this.controllers[controllerID];
    }

    validate(formData: any, schemaId: string):ValidatorResult {
        return this.validator.validate(formData,this.getSchema(schemaId));
    }
}
