import * as React from "react";
import {Skin, renderInnerComp, getNonLocalChildProps} from "./component-renderer";
import {AutoForm, autoFormSelector} from "./auto-form";
import {EntryProps} from "./form-entry";
import {Type} from "./schema-property";

const renderObjectType = (childProps:EntryProps) => {
    const key = childProps.entryKey;

    return <AutoForm repo={childProps.repo}
                     key={key}
                     entryKey={key}
                     schemaId={childProps.schemaId}
                     displayMode={childProps.displayMode}
                     formData={childProps.value || {}}
                     data-automation-id={autoFormSelector(key)}
                     onAutoViewChange={childProps.onChange as any}/>;   //TODO remove any
};

function renderArrayType(childProps:EntryProps,skin:Skin) {
    const rest = getNonLocalChildProps(childProps);
    let comps = childProps.value.map((value:any,index:number) => {
        let newAutomationId = (childProps as any)['data-automation-id']+'_'+index;

        const newOnChange = (e: React.FormEvent<HTMLInputElement>) => {
            if (childProps.onChange) {
                let newArray = childProps.value.slice(0);
                newArray[index] = e.currentTarget.value;
                let newEvent = {
                    currentTarget:{
                        value: newArray
                    }
                }as any;
                childProps.onChange(newEvent)
            }
        };

        const newChildProps:any = Object.assign({},childProps,{['data-automation-id']:newAutomationId},{onChange:newOnChange});

        const innerComp = renderInnerComp(value, skin,newChildProps);
        return <li data-automation-id={newAutomationId+'_li'} key={index}>
            {innerComp}
        </li>;
    });

    return <ol {...rest}>
        {comps}
    </ol>;
}


export class ViewSkin implements Skin {

    private renderBooleanForView = (childProps:EntryProps):JSX.Element => {
        const rest = getNonLocalChildProps(childProps);
        let newChildProps = Object.assign(rest, {
            type: "checkbox",
            disabled:true,
            checked:childProps.value,
        });
        return <input {...newChildProps}/>;
    };

    private renderNonBooleanForView = (childProps:EntryProps):JSX.Element => {
        const rest = getNonLocalChildProps(childProps);
        return <label {...rest}>{childProps.value}</label>;
    };

    boolean: (childProps: {}) => JSX.Element = this.renderBooleanForView;
    string: (childProps: {}) => JSX.Element = this.renderNonBooleanForView;
    number: (childProps: {}) => JSX.Element = this.renderNonBooleanForView;
    array: (childProps: {}) => JSX.Element = (childProps: EntryProps) =>renderArrayType(childProps,this);
    object: (childProps: {}) => JSX.Element = renderObjectType;
}

let getValueFromNativeEvent = function (type: Type, e: any) {
    let newValue:any;
    switch (type) {
        case 'boolean':
            newValue = e.target.checked;
            break;
        case 'number':
            newValue = parseInt(e.currentTarget.value);
            break;
        default:
            newValue = e.target.value;
    }
    return newValue;
};

export class EditSkin implements Skin {
    onChangeByType(type: Type,onChange:any): any {
        return (e: any) => {
            e.stopPropagation();

            const newEvent:any = {
                currentTarget: {
                    value: getValueFromNativeEvent(type, e)
                }
            };

            onChange(newEvent);
        };
    }

    constructor() {}

    private renderAnyTypeForEdit = (childProps:EntryProps ,type: Type, nativeTypeOverride?:string) => {
        const rest = getNonLocalChildProps(childProps);
        let newChildProps = Object.assign(rest, {
            onChange: this.onChangeByType(type,childProps.onChange),
            type: nativeTypeOverride || type});
        return <input {...newChildProps}/>;
    };

    private renderBooleanTypeForEdit = (childProps:EntryProps) => {
        const newChildProps = Object.assign({},
            childProps,
            {checked:childProps.value}
            );
        return this.renderAnyTypeForEdit(newChildProps,'boolean','checkbox');
    };

    boolean: (childProps: {}) => JSX.Element = this.renderBooleanTypeForEdit;
    string: (childProps: {}) => JSX.Element = (childProps: EntryProps) => this.renderAnyTypeForEdit(childProps,"string");
    number: (childProps: {}) => JSX.Element = (childProps: EntryProps) => this.renderAnyTypeForEdit(childProps,"number");
    array: (childProps: {}) => JSX.Element = (childProps: EntryProps) => renderArrayType(childProps,this);
    object: (childProps: {}) => JSX.Element = renderObjectType;
}
